﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Text;

namespace HeuristicLab.Modeling.SQLiteBackend {
  [Table(Name = "Model")]
  public class Model {
    public Model() {
      targetVariable = default(EntityRef<Variable>);
      algorithm = default(EntityRef<Algorithm>);
    }

    public Model(Variable targetVariable, Algorithm algorithm, object data)
      : this() {
      this.TargetVariable = targetVariable;
      this.Algorithm = algorithm;
      this.data = data;
    }

    private int id;
    [Column(Storage = "id", IsPrimaryKey = true, IsDbGenerated = true)]
    public int Id {
      get { return this.id; }
      private set { this.id = value; }
    }

    private object data;
    [Column(Storage = "data", DbType = "varbinary(8000)", CanBeNull = false)]
    public object Data {
      get { return this.data; }
      set { this.data = value; }
    }

    private int algorithmId;
    [Column(Storage = "algorithmId", CanBeNull = false)]
    public int AlgorithmId {
      get { return this.algorithmId; }
      private set {
        if (algorithmId != value) {
          if (algorithm.HasLoadedOrAssignedValue)
            throw new ForeignKeyReferenceAlreadyHasValueException();
          algorithmId = value;
        }
      }
    }

    private EntityRef<Algorithm> algorithm;
    [Association(Storage = "algorithm", ThisKey = "AlgorithmId", OtherKey = "Id", IsForeignKey = true)]
    public Algorithm Algorithm {
      get { return this.algorithm.Entity; }
      set {
        Algorithm previousValue = algorithm.Entity;
        if (previousValue != value || (!algorithm.HasLoadedOrAssignedValue)) {
          if (previousValue != null) {
            algorithm.Entity = null;
          }
          algorithm.Entity = value;
          if (value != null) {
            algorithmId = value.Id;
          }
        }
      }
    }

    private int targetVariableId;
    [Column(Storage = "targetVariableId", CanBeNull = false)]
    public int TargetVariableId {
      get { return this.targetVariableId; }
      private set {
        if (targetVariableId != value) {
          if (targetVariable.HasLoadedOrAssignedValue)
            throw new ForeignKeyReferenceAlreadyHasValueException();
          targetVariableId = value;
        }
      }
    }

    private EntityRef<Variable> targetVariable;
    [Association(Storage = "targetVariable", ThisKey = "TargetVariableId", OtherKey = "Id", IsForeignKey = true)]
    public Variable TargetVariable {
      get { return this.targetVariable.Entity; }
      set {
        Variable previousValue = targetVariable.Entity;
        if (previousValue != value || (!targetVariable.HasLoadedOrAssignedValue)) {
          if (previousValue != null) {
            targetVariable.Entity = null;
          }
          targetVariable.Entity = value;
          if (value != null) {
            targetVariableId = value.Id;
          } 
        }
      }
    }

    public IEnumerable<Variable> InputVariables {
      get {
        using (ModelingDataContext ctx = new ModelingDataContext()) {
          var x = from inputVariable in ctx.InputVariables
                  where inputVariable.Model == this
                  select inputVariable.Variable;
          return x.AsEnumerable();
        }
      }
    }

    public Dictionary<string, double> ResultValues {
      get {
        using (ModelingDataContext ctx = new ModelingDataContext()) {
          var x = (from modelResult in ctx.ModelResults
                   where modelResult.Model == this
                   select modelResult)
                  .ToDictionary(
                    modelResult => modelResult.Result.Name,
                    modelResult => modelResult.Value);
          return x;
        }
      }
    }
  }
}
